@@ -3,12 +3,14 @@ import lxml.etree as ET
 
 from . import account_aliases_fetcher
 from . import _duo_authenticator as duo_auth
+from . import _duo_universal_prompt_authenticator as duo_universal_prompt_auth
 from . import _rsa_authenticator as rsa_auth
 from . import _symantec_vip_access as symantec_vip_access
 from . import _azure_mfa_authenticator as azure_mfa_auth
 from . import _azure_cloud_mfa_authenticator as azure_cloud_mfa_auth
 from . import html_roles_fetcher
 from . import roles_assertion_extractor
+from .helpers import trace_http_request
 
 
 def authenticate(config, username=None, password=None, assertfile=None):
@@ -22,6 +24,7 @@ def authenticate(config, username=None, password=None, assertfile=None):
         password=password,
         sspi=config.sspi,
     )
+    trace_http_request(response)
 
     assertion = None
     aws_session_duration = None
@@ -33,21 +36,7 @@ def authenticate(config, username=None, password=None, assertfile=None):
         principal_roles, assertion, aws_session_duration = extract_strategy()
 
         if assertion is None:
-            logging.debug(u'''Cannot extract saml assertion from request's response. Re-authentication needed?:
-                * url: {}
-                * headers: {}
-            Response:
-                * status: {}
-                * headers: {}
-                * body: {}
-            '''.format(
-                response.url,
-                response.request.headers,
-                response.status_code,
-                response.headers,
-                response.text
-            ))
-            logging.error(u'Cannot extract saml assertion. Re-authentication needed?')
+            logging.error("Cannot extract saml assertion from request's response. Re-authentication needed?")
         else:
             aggregated_principal_roles = _aggregate_roles_by_account_alias(session,
                                                                            config,
@@ -57,20 +46,6 @@ def authenticate(config, username=None, password=None, assertfile=None):
                                                                            principal_roles)
 
     else:
-        logging.debug(u'''Cannot extract roles from request's response:
-                * url: {}
-                * headers: {}
-            Response:
-                * status: {}
-                * headers: {}
-                * body: {}
-            '''.format(
-            response.url,
-            response.request.headers,
-            response.status_code,
-            response.headers,
-            response.text
-        ))
         logging.error(u'Cannot extract roles from response')
 
     logging.debug(u'Roles along with principals found after authentication: {}'.format(aggregated_principal_roles))
@@ -113,6 +88,14 @@ def _strategy(response, config, session, assertfile=None):
             return duo_auth.extract(html_response, config.ssl_verification, session)
         return extract
 
+    def _duo_universal_prompt_extractor():
+        def extract():
+            return duo_universal_prompt_auth.extract(
+                html_response, config.ssl_verification, session, config.duo_factor, config.duo_device
+            )
+
+        return extract
+
     def _symantec_vip_extractor():
         def extract():
             return symantec_vip_access.extract(html_response, config.ssl_verification, session)
@@ -145,6 +128,8 @@ def _strategy(response, config, session, assertfile=None):
 
     if _is_duo_authentication(html_response):
         chosen_strategy = _duo_extractor
+    elif _is_duo_universal_prompt_authentication(html_response):
+        chosen_strategy = _duo_universal_prompt_extractor
     elif _is_symantec_vip_authentication(html_response):
         chosen_strategy = _symantec_vip_extractor
     elif _is_rsa_authentication(html_response):
@@ -161,9 +146,18 @@ def _is_duo_authentication(html_response):
     duo_auth_method = './/input[@id="authMethod"]'
     element = html_response.find(duo_auth_method)
     duo = element is not None
-    duo = duo and element.get('value') == 'DuoAdfsAdapter'
+    duo = duo and element.get("value") == "DuoAdfsAdapter"
     return duo
 
+
+def _is_duo_universal_prompt_authentication(html_response):
+    duo_auth_method = './/form[@id="adfs_form"]/input[@name="adfs_auth_method"]'
+    element = html_response.find(duo_auth_method)
+    duo = element is not None
+    duo = duo and element.get("value") == "DuoAdfsAdapter"
+    return duo
+
+
 def _is_symantec_vip_authentication(html_response):
     auth_method = './/input[@id="authMethod"]'
     element = html_response.find(auth_method)
